@@ -71,6 +71,8 @@ from .types import (  # noqa
     DateTimeRangeType,
     EmailType,
     EncryptedType,
+    EnrichedDateTimeType,
+    EnrichedDateType,
     instrumented_list,
     InstrumentedList,
     Int8RangeType,
@@ -101,4 +103,4 @@ from .view import (  # noqa
     refresh_materialized_view
 )
 
-__version__ = '0.36.4'
+__version__ = '0.36.5'
