@@ -1,4 +1,7 @@
-import collections
+try:
+    from collections.abc import Mapping, Sequence
+except ImportError:  # For python 2.7 support
+    from collections import Mapping, Sequence
 import itertools
 import os
 from copy import copy
@@ -7,92 +10,10 @@ import sqlalchemy as sa
 from sqlalchemy.engine.url import make_url
 from sqlalchemy.exc import OperationalError, ProgrammingError
 
-from ..expressions import explain_analyze
 from ..utils import starts_with
 from .orm import quote
 
 
-class PlanAnalysis(object):
-    def __init__(self, plan):
-        self.plan = plan
-
-    @property
-    def node_types(self):
-        types = [self.plan['Node Type']]
-        if 'Plans' in self.plan:
-            for plan in self.plan['Plans']:
-                analysis = PlanAnalysis(plan)
-                types.extend(analysis.node_types)
-        return types
-
-
-class QueryAnalysis(object):
-    def __init__(self, result_set):
-        self.plan = result_set[0]['Plan']
-        if 'Total Runtime' in result_set[0]:
-            # PostgreSQL versions < 9.4
-            self.runtime = result_set[0]['Total Runtime']
-        else:
-            # PostgreSQL versions >= 9.4
-            self.runtime = (
-                result_set[0]['Execution Time'] +
-                result_set[0]['Planning Time']
-            )
-
-    @property
-    def node_types(self):
-        return list(PlanAnalysis(self.plan).node_types)
-
-    def __repr__(self):
-        return '<QueryAnalysis runtime=%r>' % self.runtime
-
-
-def analyze(conn, query):
-    """
-    Analyze query using given connection and return :class:`QueryAnalysis`
-    object. Analysis is performed using database specific EXPLAIN ANALYZE
-    construct and then examining the results into structured format. Currently
-    only PostgreSQL is supported.
-
-
-    Getting query runtime (in database level) ::
-
-
-        from sqlalchemy_utils import analyze
-
-
-        analysis = analyze(conn, 'SELECT * FROM article')
-        analysis.runtime  # runtime as milliseconds
-
-
-    Analyze can be very useful when testing that query doesn't issue a
-    sequential scan (scanning all rows in table). You can for example write
-    simple performance tests this way.::
-
-
-        query = (
-            session.query(Article.name)
-            .order_by(Article.name)
-            .limit(10)
-        )
-        analysis = analyze(self.connection, query)
-        analysis.node_types  # [u'Limit', u'Index Only Scan']
-
-        assert 'Seq Scan' not in analysis.node_types
-
-
-    .. versionadded: 0.26.17
-
-    :param conn: SQLAlchemy Connection object
-    :param query: SQLAlchemy Query object or query as a string
-    """
-    return QueryAnalysis(
-        conn.execute(
-            explain_analyze(query, buffers=True, format='json')
-        ).scalar()
-    )
-
-
 def escape_like(string, escape_char='*'):
     """
     Escape the string paremeter used in SQL LIKE expressions.
@@ -167,7 +88,7 @@ def json_sql(value, scalars_to_json=True):
         def scalar_convert(a):
             return sa.func.to_json(sa.text(a))
 
-    if isinstance(value, collections.Mapping):
+    if isinstance(value, Mapping):
         return sa.func.json_build_object(
             *(
                 json_sql(v, scalars_to_json=False)
@@ -176,7 +97,7 @@ def json_sql(value, scalars_to_json=True):
         )
     elif isinstance(value, str):
         return scalar_convert("'{0}'".format(value))
-    elif isinstance(value, collections.Sequence):
+    elif isinstance(value, Sequence):
         return sa.func.json_build_array(
             *(
                 json_sql(v, scalars_to_json=False)
@@ -188,6 +109,78 @@ def json_sql(value, scalars_to_json=True):
     return value
 
 
+def jsonb_sql(value, scalars_to_jsonb=True):
+    """
+    Convert python data structures to PostgreSQL specific SQLAlchemy JSONB
+    constructs. This function is extremly useful if you need to build
+    PostgreSQL JSONB on python side.
+
+    .. note::
+
+        This function needs PostgreSQL >= 9.4
+
+    Scalars are converted to to_jsonb SQLAlchemy function objects
+
+    ::
+
+        jsonb_sql(1)     # Equals SQL: to_jsonb(1)
+
+        jsonb_sql('a')   # to_jsonb('a')
+
+
+    Mappings are converted to jsonb_build_object constructs
+
+    ::
+
+        jsonb_sql({'a': 'c', '2': 5})  # jsonb_build_object('a', 'c', '2', 5)
+
+
+    Sequences (other than strings) converted to jsonb_build_array constructs
+
+    ::
+
+        jsonb_sql([1, 2, 3])  # jsonb_build_array(1, 2, 3)
+
+
+    You can also nest these data structures
+
+    ::
+
+        jsonb_sql({'a': [1, 2, 3]})
+        # jsonb_build_object('a', jsonb_build_array[1, 2, 3])
+
+
+    :param value:
+        value to be converted to SQLAlchemy PostgreSQL function constructs
+    :boolean jsonbb:
+        Flag to alternatively convert the return with a to_jsonb construct
+    """
+    scalar_convert = sa.text
+    if scalars_to_jsonb:
+        def scalar_convert(a):
+            return sa.func.to_jsonb(sa.text(a))
+
+    if isinstance(value, Mapping):
+        return sa.func.jsonb_build_object(
+            *(
+                jsonb_sql(v, scalars_to_jsonb=False)
+                for v in itertools.chain(*value.items())
+            )
+        )
+    elif isinstance(value, str):
+        return scalar_convert("'{0}'".format(value))
+    elif isinstance(value, Sequence):
+        return sa.func.jsonb_build_array(
+            *(
+                jsonb_sql(v, scalars_to_jsonb=False)
+                for v in value
+            )
+        )
+    elif isinstance(value, (int, float)):
+        return scalar_convert(str(value))
+    return value
+
+
 def has_index(column_or_constraint):
     """
     Return whether or not given column or the columns of given foreign key
@@ -438,23 +431,39 @@ def database_exists(url):
     Performs backend-specific testing to quickly determine if a database
     exists on the server. ::
 
-        database_exists('postgres://postgres@localhost/name')  #=> False
-        create_database('postgres://postgres@localhost/name')
-        database_exists('postgres://postgres@localhost/name')  #=> True
+        database_exists('postgresql://postgres@localhost/name')  #=> False
+        create_database('postgresql://postgres@localhost/name')
+        database_exists('postgresql://postgres@localhost/name')  #=> True
 
     Supports checking against a constructed URL as well. ::
 
-        engine = create_engine('postgres://postgres@localhost/name')
+        engine = create_engine('postgresql://postgres@localhost/name')
         database_exists(engine.url)  #=> False
         create_database(engine.url)
         database_exists(engine.url)  #=> True
 
     """
 
+    def get_scalar_result(engine, sql):
+        result_proxy = engine.execute(sql)
+        result = result_proxy.scalar()
+        result_proxy.close()
+        engine.dispose()
+        return result
+
+    def sqlite_file_exists(database):
+        if not os.path.isfile(database) or os.path.getsize(database) < 100:
+            return False
+
+        with open(database, 'rb') as f:
+            header = f.read(100)
+
+        return header[:16] == b'SQLite format 3\x00'
+
     url = copy(make_url(url))
     database = url.database
-    if url.drivername.startswith('postgresql'):
-        url.database = 'template1'
+    if url.drivername.startswith('postgres'):
+        url.database = 'postgres'
     else:
         url.database = None
 
@@ -462,31 +471,37 @@ def database_exists(url):
 
     if engine.dialect.name == 'postgresql':
         text = "SELECT 1 FROM pg_database WHERE datname='%s'" % database
-        return bool(engine.execute(text).scalar())
+        return bool(get_scalar_result(engine, text))
 
     elif engine.dialect.name == 'mysql':
         text = ("SELECT SCHEMA_NAME FROM INFORMATION_SCHEMA.SCHEMATA "
                 "WHERE SCHEMA_NAME = '%s'" % database)
-        return bool(engine.execute(text).scalar())
+        return bool(get_scalar_result(engine, text))
 
     elif engine.dialect.name == 'sqlite':
         if database:
-            return database == ':memory:' or os.path.exists(database)
+            return database == ':memory:' or sqlite_file_exists(database)
         else:
             # The default SQLAlchemy database is in memory,
             # and :memory is not required, thus we should support that use-case
             return True
 
     else:
+        engine.dispose()
+        engine = None
         text = 'SELECT 1'
         try:
             url.database = database
             engine = sa.create_engine(url)
-            engine.execute(text)
+            result = engine.execute(text)
+            result.close()
             return True
 
         except (ProgrammingError, OperationalError):
             return False
+        finally:
+            if engine is not None:
+                engine.dispose()
 
 
 def create_database(url, encoding='utf8', template=None):
@@ -501,7 +516,7 @@ def create_database(url, encoding='utf8', template=None):
     To create a database, you can pass a simple URL that would have
     been passed to ``create_engine``. ::
 
-        create_database('postgres://postgres@localhost/name')
+        create_database('postgresql://postgres@localhost/name')
 
     You may also pass the url from an existing engine. ::
 
@@ -515,12 +530,20 @@ def create_database(url, encoding='utf8', template=None):
 
     database = url.database
 
-    if url.drivername.startswith('postgresql'):
-        url.database = 'template1'
+    if url.drivername.startswith('postgres'):
+        url.database = 'postgres'
+    elif url.drivername.startswith('mssql'):
+        url.database = 'master'
     elif not url.drivername.startswith('sqlite'):
         url.database = None
 
-    engine = sa.create_engine(url)
+    if url.drivername == 'mssql+pyodbc':
+        engine = sa.create_engine(url, connect_args={'autocommit': True})
+    elif url.drivername == 'postgresql+pg8000':
+        engine = sa.create_engine(url, isolation_level='AUTOCOMMIT')
+    else:
+        engine = sa.create_engine(url)
+    result_proxy = None
 
     if engine.dialect.name == 'postgresql':
         if engine.driver == 'psycopg2':
@@ -530,29 +553,34 @@ def create_database(url, encoding='utf8', template=None):
             )
 
         if not template:
-            template = 'template0'
+            template = 'template1'
 
         text = "CREATE DATABASE {0} ENCODING '{1}' TEMPLATE {2}".format(
             quote(engine, database),
             encoding,
             quote(engine, template)
         )
-        engine.execute(text)
+        result_proxy = engine.execute(text)
 
     elif engine.dialect.name == 'mysql':
         text = "CREATE DATABASE {0} CHARACTER SET = '{1}'".format(
             quote(engine, database),
             encoding
         )
-        engine.execute(text)
+        result_proxy = engine.execute(text)
 
     elif engine.dialect.name == 'sqlite' and database != ':memory:':
         if database:
-            open(database, 'w').close()
+            engine.execute("CREATE TABLE DB(id int);")
+            engine.execute("DROP TABLE DB;")
 
     else:
         text = 'CREATE DATABASE {0}'.format(quote(engine, database))
-        engine.execute(text)
+        result_proxy = engine.execute(text)
+
+    if result_proxy is not None:
+        result_proxy.close()
+    engine.dispose()
 
 
 def drop_database(url):
@@ -563,7 +591,7 @@ def drop_database(url):
     Works similar to the :ref:`create_database` method in that both url text
     and a constructed url are accepted. ::
 
-        drop_database('postgres://postgres@localhost/name')
+        drop_database('postgresql://postgres@localhost/name')
         drop_database(engine.url)
 
     """
@@ -572,12 +600,20 @@ def drop_database(url):
 
     database = url.database
 
-    if url.drivername.startswith('postgresql'):
-        url.database = 'template1'
+    if url.drivername.startswith('postgres'):
+        url.database = 'postgres'
+    elif url.drivername.startswith('mssql'):
+        url.database = 'master'
     elif not url.drivername.startswith('sqlite'):
         url.database = None
 
-    engine = sa.create_engine(url)
+    if url.drivername == 'mssql+pyodbc':
+        engine = sa.create_engine(url, connect_args={'autocommit': True})
+    elif url.drivername == 'postgresql+pg8000':
+        engine = sa.create_engine(url, isolation_level='AUTOCOMMIT')
+    else:
+        engine = sa.create_engine(url)
+    conn_resource = None
 
     if engine.dialect.name == 'sqlite' and database != ':memory:':
         if database:
@@ -605,6 +641,11 @@ def drop_database(url):
         # Drop the database.
         text = 'DROP DATABASE {0}'.format(quote(connection, database))
         connection.execute(text)
+        conn_resource = connection
     else:
         text = 'DROP DATABASE {0}'.format(quote(engine, database))
-        engine.execute(text)
+        conn_resource = engine.execute(text)
+
+    if conn_resource is not None:
+        conn_resource.close()
+    engine.dispose()
